@@ -108,6 +108,8 @@ private:
         TEST_CASE(canFindMatchingBracketsWithTooManyOpening);
         TEST_CASE(findClosingBracket);
         TEST_CASE(findClosingBracket2);
+        TEST_CASE(findClosingBracket3);
+        TEST_CASE(findClosingBracket4);
 
         TEST_CASE(expressionString);
 
@@ -118,17 +120,17 @@ private:
         TokensFrontBack tokensFrontBack(list);
         auto *token = new Token(tokensFrontBack);
         token->str("1");
-        token->insertToken("2");
-        token->next()->insertToken("3");
+        (void)token->insertToken("2");
+        (void)token->next()->insertToken("3");
         Token *last = token->tokAt(2);
         ASSERT_EQUALS(token->str(), "1");
-        ASSERT_EQUALS(token->next()->str(), "2");
+        ASSERT_EQUALS(token->strAt(1), "2");
         // cppcheck-suppress redundantNextPrevious - this is intentional
         ASSERT_EQUALS(token->tokAt(2)->str(), "3");
         ASSERT_EQUALS_MSG(true, last->next() == nullptr, "Null was expected");
 
         ASSERT_EQUALS(last->str(), "3");
-        ASSERT_EQUALS(last->previous()->str(), "2");
+        ASSERT_EQUALS(last->strAt(-1), "2");
         // cppcheck-suppress redundantNextPrevious - this is intentional
         ASSERT_EQUALS(last->tokAt(-2)->str(), "1");
         ASSERT_EQUALS_MSG(true, token->previous() == nullptr, "Null was expected");
@@ -141,7 +143,7 @@ private:
         SimpleTokenizer tokenizer(settingsDefault, *this);
         const std::string code2 = ";" + code + ";";
         try {
-            ASSERT_LOC(tokenizer.tokenize(code2.c_str()), file, line);
+            ASSERT_LOC(tokenizer.tokenize(code2), file, line);
         } catch (...) {}
         return Token::Match(tokenizer.tokens()->next(), pattern.c_str(), varid);
     }
@@ -354,6 +356,30 @@ private:
         ASSERT_EQUALS(false, tok.isUtf32());
         ASSERT_EQUALS(false, tok.isLong());
         ASSERT_EQUALS(true, tok.isCMultiChar());
+
+        tok.str("'\\''");
+        ASSERT_EQUALS(true, tok.isCChar());
+        ASSERT_EQUALS(false, tok.isUtf8());
+        ASSERT_EQUALS(false, tok.isUtf16());
+        ASSERT_EQUALS(false, tok.isUtf32());
+        ASSERT_EQUALS(false, tok.isLong());
+        ASSERT_EQUALS(false, tok.isCMultiChar());
+
+        tok.str("'\\r\\n'");
+        ASSERT_EQUALS(false, tok.isCChar());
+        ASSERT_EQUALS(false, tok.isUtf8());
+        ASSERT_EQUALS(false, tok.isUtf16());
+        ASSERT_EQUALS(false, tok.isUtf32());
+        ASSERT_EQUALS(false, tok.isLong());
+        ASSERT_EQUALS(true, tok.isCMultiChar());
+
+        tok.str("'\\x10'");
+        ASSERT_EQUALS(true, tok.isCChar());
+        ASSERT_EQUALS(false, tok.isUtf8());
+        ASSERT_EQUALS(false, tok.isUtf16());
+        ASSERT_EQUALS(false, tok.isUtf32());
+        ASSERT_EQUALS(false, tok.isLong());
+        ASSERT_EQUALS(false, tok.isCMultiChar());
     }
 
     void stringTypes() const {
@@ -523,7 +549,7 @@ private:
         TokensFrontBack listEnds(list);
         Token ** const tokensBack = &(listEnds.back);
         Token tok(listEnds);
-        tok.insertToken("aba");
+        (void)tok.insertToken("aba");
         ASSERT_EQUALS(true, *tokensBack == tok.next());
         tok.deleteNext();
         ASSERT_EQUALS(true, *tokensBack == &tok);
@@ -534,7 +560,7 @@ private:
         Token ** const tokensFront = &(listEnds.front);
         Token tok(listEnds);
 
-        tok.insertToken("aba");
+        (void)tok.insertToken("aba");
 
         ASSERT_EQUALS(true, *tokensFront == tok.previous());
         tok.deletePrevious();
@@ -1157,6 +1183,24 @@ private:
         ASSERT(t && Token::simpleMatch(t->findClosingBracket(), ">"));
     }
 
+    void findClosingBracket3() {
+        const SimpleTokenizer var(*this, // #12789
+                                  "template <size_t I = 0, typename... ArgsT, std::enable_if_t<I < sizeof...(ArgsT)>* = nullptr>\n"
+                                  "void f();\n");
+        const Token* const t = Token::findsimplematch(var.tokens(), "<");
+        ASSERT(t && Token::simpleMatch(t->findClosingBracket(), ">"));
+    }
+
+    void findClosingBracket4() {
+        const SimpleTokenizer var(*this, // #12923
+                                  "template<template<class E> class T = std::vector, class U = std::vector<int>, class V = void>\n"
+                                  "class C;\n");
+        const Token *const t = Token::findsimplematch(var.tokens(), "<");
+        ASSERT(t);
+        const Token *const closing = t->findClosingBracket();
+        ASSERT(closing && closing == var.tokens()->tokAt(28));
+    }
+
     void expressionString() {
         const SimpleTokenizer var1(*this, "void f() { *((unsigned long long *)x) = 0; }");
         const Token *const tok1 = Token::findsimplematch(var1.tokens(), "*");
